@@ -34,7 +34,6 @@ typedef int ssize_t;
 #ifdef UNIXWARE1
 #define DONT_CHMOD_SOCKET
 #define O_RSYNC		0
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -80,7 +79,6 @@ typedef int ssize_t;
 #define HAVE_GETRUSAGE
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
-#define SIZEOF_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define PATH_TMP	"/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -93,7 +91,6 @@ typedef int ssize_t;
 
 #ifdef OSF1
 #define HAVE_GETRUSAGE
-#define SIZEOF_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define PATH_TMP	"/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -108,7 +105,6 @@ typedef int ssize_t;
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       socklen_t
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -122,7 +118,6 @@ typedef int ssize_t;
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       size_t
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -134,7 +129,6 @@ typedef int ssize_t;
 
 #ifdef SOLARIS_251
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -149,7 +143,6 @@ int getrusage(int who, struct rusage *rusage);
 
 #ifdef SOLARIS_OLD
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -161,7 +154,6 @@ int getrusage(int who, struct rusage *rusage);
 
 #ifdef MACOSX
 #define HAVE_GETRUSAGE
-#define SIZEOF_IS_LONG_INT
 /* not supported */
 #define O_RSYNC 0
 #define O_SYNC 0
@@ -179,7 +171,6 @@ int getrusage(int who, struct rusage *rusage);
 
 #ifdef NEXTSTEP
 #define HAVE_GETRUSAGE
-#define SIZEOF_IS_LONG_INT
 /* not supported */
 #define SA_RESTART 0
 #define O_RSYNC 0
@@ -199,7 +190,6 @@ int getrusage(int who, struct rusage *rusage);
 
 #ifdef IRIX62
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -213,7 +203,6 @@ int getrusage(int who, struct rusage *rusage);
 #define HAVE_GETRUSAGE
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -223,13 +212,23 @@ int getrusage(int who, struct rusage *rusage);
 #define PATH_SYSLOG     "/var/adm/SYSLOG"
 #endif
 
-#if defined(_AIX43) || defined (AIX43)
+#ifdef AIX32
+extern void closelog();
 #define HAVE_GETRUSAGE
-#if defined(__64BIT_KERNEL) && defined(_KERNEL)
-#define PID_T_IS_LONG_INT
-#endif /* __64BIT_KERNEL && _KERNEL */
-#define SIZEOF_IS_LONG_INT
-#define SOCKLEN_T	size_t
+#define NEED_SYS_SELECT_H
+#define SOCKLEN_T       int
+#define O_RSYNC         0
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP    "/var/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/var/adm/wtmp"
+#define PATH_UTMP       "/etc/utmp"
+#define PATH_SYSLOG     "/var/spool/mqueue/syslog"
+#endif
+
+#ifdef AIX43
+#define HAVE_GETRUSAGE
+#define SOCKLEN_T	socklen_t
 #define PATH_TMP	"/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
 #define PATH_PASSWD	"/etc/passwd"
@@ -239,13 +238,9 @@ int getrusage(int who, struct rusage *rusage);
 #define PATH_MAILLOG	"/var/spool/mqueue/syslog"
 #endif
 
-#if defined(_AIX41) || defined (AIX41)
+#ifdef AIX41
 #define HAVE_GETRUSAGE
-#if defined(__64BIT_KERNEL) && defined(_KERNEL)
-#define PID_T_IS_LONG_INT
-#endif /* __64BIT_KERNEL && _KERNEL */
 #define NEED_SYS_SELECT_H
-#define SIZEOF_IS_LONG_INT
 #define SOCKLEN_T	int
 #define O_RSYNC		0
 #define PATH_TMP	"/tmp"
@@ -255,13 +250,28 @@ int getrusage(int who, struct rusage *rusage);
 #define PATH_UTMP	"/etc/utmp"
 #define PATH_SYSLOG	"/var/adm/messages"
 #define PATH_MAILLOG	"/var/spool/mqueue/syslog"
-#endif /* _AIX41 || AIX41 */
+#endif
+
+#if defined(ULTRIX)
+typedef int     ssize_t;
+extern void closelog(void);
+#include <sys/syslog.h>
+#define HAVE_GETRUSAGE
+#define SOCKLEN_T       int
+#define SA_RESTART      0
+#define O_RSYNC         0
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP    "/var/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/var/adm/wtmp"
+#define PATH_UTMP       "/etc/utmp"
+#define PATH_SYSLOG     "/var/spool/mqueue/syslog"
+#endif
 
 #ifdef UNIXWARE7
 #define HAVE_GETRUSAGE
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define O_RSYNC		0
 #define PATH_TMP        "/tmp"
@@ -281,7 +291,6 @@ int getrusage(int who, struct rusage *rusage);
 #define HAVE_GETRUSAGE
 #define HAVE_SNPRINTF
 #define HAVE_VSNPRINTF
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define O_RSYNC		0
 #define PATH_TMP        "/tmp"
@@ -314,7 +323,11 @@ extern void closelog(void);
 #define PATH_WTMP       "/var/adm/wtmp"
 #define PATH_UTMP       "/etc/utmp"
 #define PATH_SYSLOG     "/var/adm/syslog"
+#ifdef S_ISSOCK
 #define IS_UNIX_SOCKET(m)  (S_ISSOCK(m) || S_ISFIFO(m))
+#else
+#define IS_UNIX_SOCKET(m)  S_ISFIFO(m)
+#endif
 #endif
 
 #ifdef SUNOS4
@@ -379,7 +392,6 @@ extern closelog(void);
 #undef HAVE_SNPRINTF
 #undef HAVE_VSNPRINTF
 #define HAVE_GETRUSAGE
-#define PID_T_IS_LONG_INT
 #define SOCKLEN_T       size_t
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
@@ -390,6 +402,29 @@ extern closelog(void);
 #define IS_UNIX_SOCKET(m)  (S_ISSOCK(m) || S_ISFIFO(m))
 #endif
 
+#ifdef UNICOS
+#define HAVE_DAEMON
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
+#define O_RSYNC         0
+#define SA_RESTART 0
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP    "/usr/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/etc/wtmp"
+#define PATH_UTMP       "/etc/utmp"
+/* System logging on Unicos is different, so the following settings probably
+ * are not perfect. */
+#define PATH_SYSLOG "/usr/adm/syslog/daylog"
+#define PATH_MAILLOG "/usr/spool/mqueue/syslog"
+#endif
+
+#ifdef TANDEM
+#define O_RSYNC         0
+#define SA_RESTART      0
+#define IS_UNIX_SOCKET(m)  (S_ISSOCK(m) || S_ISFIFO(m))
+#endif
+
 #ifndef RANDSAVENAME
 #define RANDSAVENAME	"/etc/prngd-seed"
 #endif
