@@ -25,7 +25,8 @@ static volatile int received_sighup = 0;
 typedef enum
 {
   wait_command, wait_second_byte, wait_third_byte, wait_fourth_byte,
-  wait_read, wait_write, wait_entropy, to_be_closed, to_be_error_closed
+  wait_read, wait_entropy1, wait_entropy2, wait_write, to_be_closed,
+  to_be_error_closed
 }
 querystate_t;
 
@@ -41,21 +42,56 @@ typedef struct
 } egd_query_t;
 
 
-static void termination_handler(int received_signal /*ARGSUSED*/)
+static void termination_handler(int received_signal)
 {
+  char msg_buf[128];
+
+  if (received_signal != SIGTERM) {
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"Termination handler called with wrong signal: %d != SIGTERM",
+	received_signal);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_notice(msg_buf, 0);
+  }
   if (debug)
     (void)fprintf(stderr, "Caught SIGTERM, shutting down\n");
   close_seedfile();
+  if (pidfile) {
+    if (remove(pidfile))
+      (void)snprintf(msg_buf, sizeof(msg_buf), "Could not remove pidfile %s",
+		     pidfile);
+      msg_buf[sizeof(msg_buf) - 1] = '\0';
+      error_error(msg_buf, errno);
+      exit(EXIT_FAILURE);
+  }
   exit(EXIT_SUCCESS);
 }
 
-static void sighup_handler(int received_signal /*ARGSUSED*/)
+static void sighup_handler(int received_signal)
 {
+  char msg_buf[128];
+
+  if (received_signal != SIGHUP) {
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"Termination handler called with wrong signal: %d != SIGHUP",
+	received_signal);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_notice(msg_buf, 0);
+  }
   received_sighup = 1;
 }
 
-static void child_handler(int received_signal /*ARGSUSED*/)
+static void child_handler(int received_signal)
 {
+  char msg_buf[128];
+
+  if (received_signal != SIGCHLD) {
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"Termination handler called with wrong signal: %d != SIGCHLD",
+	received_signal);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_notice(msg_buf, 0);
+  }
   /*
    * Wait for child process to avoid zombies. This signal comes in
    * asynchronously.
@@ -73,15 +109,15 @@ static void child_handler(int received_signal /*ARGSUSED*/)
   caught_sigchild = 1;
 }
 
-static void collect_children(int *gatherer_pid)
+static void collect_children(pid_t *gatherer_pid)
 {
-  int child_pid;
+  pid_t child_pid;
   int gatherer_status;
 
   while (((child_pid = waitpid(-1, &gatherer_status, WNOHANG)) > 0)
 	 || (errno == EINTR)) {
     if (child_pid > 0) {
-      rand_add((int *)&gatherer_status, sizeof(gatherer_status), 0.0);
+      rand_add(&gatherer_status, sizeof(gatherer_status), 0.0);
       if (WIFEXITED(gatherer_status) || WIFSIGNALED(gatherer_status)) {
         if (debug)
 	  (void)fprintf(stderr,
@@ -99,7 +135,7 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	       time_t seed_ext_interval, entropy_source_t *entropy_source,
 	       int max_gatherer_bytes)
 {
-  int i;
+  int i, i1;
   int accept_fd;
   struct sockaddr addr;
   SOCKLEN_T addrlen;
@@ -108,6 +144,7 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
   time_t last_seed_stat = 0, last_seed_ext = 0;
   int max_query = 0;
   int alloc_query = 0;
+  int prev_location = 0;
   egd_query_t *egd_query = NULL;
   egd_query_t *egd_query_old;
   int max_fd;
@@ -117,7 +154,7 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
   int gatherer_entry;
   int gatherer_bytes;
   int first_gather = 1;
-  int retrieved_bytes = 0;
+  int entropy_required;
   double est = 0.0;
   time_t gather_start;
   unsigned char gatherbuf[10240];
@@ -193,6 +230,7 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
       if (gather_fd > max_fd)
 	max_fd = gather_fd;
     }
+    entropy_required = 0;
     for (i = 0; i < max_query; i++)
     {
       if (egd_query[i].fd > max_fd)
@@ -204,18 +242,24 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	  (egd_query[i].state == wait_fourth_byte) ||
 	  (egd_query[i].state == wait_read))
 	FD_SET(egd_query[i].fd, &rfds);
-      else
+      else if (egd_query[i].state == wait_write)
 	FD_SET(egd_query[i].fd, &wfds);
-
+      else if ((egd_query[i].state == wait_entropy1) ||
+	       (egd_query[i].state == wait_entropy2))
+        entropy_required = 1;
     }
+
     /*
      * If we just entered the loop and gather for the first time, we
-     * don't want to wait.
+     * don't want to wait. We also want to refill our pool when drained.
+     * If there are connections waiting for random numbers to be generated,
+     * go ahead.
      */
-    if (first_gather || (rand_pool() < (ENTROPY_THRESHOLD)) || retrieved_bytes)
+    if (first_gather || (rand_pool() < (ENTROPY_THRESHOLD)) ||
+        (entropy_required && (rand_pool() > 0)))
     {
       timeout.tv_sec = 0;
-      timeout.tv_usec = 10;
+      timeout.tv_usec = 1;
     }
     else {
       timeout.tv_sec = (seed_stat_interval < seed_ext_interval ?
@@ -300,6 +344,7 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	    (void)close(accept_fd);	/*
 				 * Better this than complete shut down 
 				 */
+	    count_openfd--;
 	    egd_query = egd_query_old;
 	    continue;
 	  }
@@ -491,17 +536,11 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
      * the loop only over the sockets already open, as the data returned
      * from select() obviously cannot make sense for the connections
      * freshly opened.
-     * Service is in the sequence the fds were opened, but there is one
-     * exception: during the generation of the random numbers inside the
-     * PRNG, no other operation can be performed. It is therefore not
-     * good to generate too much random numbers inside the loop without
-     * actually leaving the loop. Incoming connections may pend on the
-     * listen socket. We will thus leave the loop as soon as we retrieved
-     * random numbers from the PNRG and run through the complete serverloop
-     * once again.
+     * Service is in the sequence the fds were opened. The generation of
+     * random numbers is performed in a seperate loop later on, as this
+     * takes some time. Service all available I/O operations first.
      */
-    for (i = retrieved_bytes = 0;
-	 (i < max_query_old) && (!retrieved_bytes); i++)
+    for (i = 0; i < max_query_old; i++)
     {
       if (FD_ISSET(egd_query[i].fd, &efds))
       {
@@ -511,18 +550,9 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	egd_query[i].state = to_be_error_closed;
       }
       else if (FD_ISSET(egd_query[i].fd, &rfds) ||
-	       FD_ISSET(egd_query[i].fd, &wfds) ||
-	       (egd_query[i].state == wait_entropy)) {
+	       FD_ISSET(egd_query[i].fd, &wfds)) {
       switch (egd_query[i].state)
       {
-      case wait_entropy:
-	  seed_internal(NULL);	/* Mix in just before query */
-	  seed_stat();
-	  if (rand_bytes(egd_query[i].buffer, egd_query[i].secbyte) > 0) {
-	    egd_query[i].state = wait_write;
-	    retrieved_bytes = 1;
-	  }
-	break;
       case wait_command:
 	error = socket_read(egd_query[i].fd, &(egd_query[i].command), 1);
 	if (error == -1)
@@ -549,7 +579,10 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	else if (egd_query[i].command == 4)
 	{
 	  egd_query[i].buffer[0] = 12;
-	  (void)sprintf((char *) (egd_query[i].buffer + 1), "%12ld", (long)getpid());
+	  (void)snprintf((char *) (egd_query[i].buffer + 1), 
+			 sizeof(egd_query[i].buffer) - 1, "%12ld",
+			 (long)getpid());
+	  egd_query[i].buffer[sizeof(egd_query[i].buffer) - 1] = '\0';
 	  egd_query[i].to_process = 13;
 	  egd_query[i].yet_processed = 0;
 	  egd_query[i].state = wait_write;
@@ -563,29 +596,27 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	{
 	  seed_internal(NULL);	/* Mix in just before query */
 	  seed_stat();
-	  if (rand_bytes(egd_query[i].buffer + 1, egd_query[i].secbyte)) {
-	    numbytes = egd_query[i].secbyte;
-	    retrieved_bytes = 1;
+	  if (rand_pool() > 0) {
+	    egd_query[i].buffer[0] = (char)egd_query[i].secbyte;
+	    egd_query[i].to_process = egd_query[i].secbyte + 1;
+            egd_query[i].yet_processed = 0;
+            egd_query[i].state = wait_entropy1;
 	  }
-	  else
+	  else {
 	    numbytes = 0;
-	  egd_query[i].buffer[0] = (char)numbytes;
-	  egd_query[i].to_process = numbytes + 1;
-	  egd_query[i].yet_processed = 0;
-	  egd_query[i].state = wait_write;
+	    egd_query[i].buffer[0] = (char)0;
+	    egd_query[i].to_process =  1;
+	    egd_query[i].yet_processed = 0;
+	    egd_query[i].state = wait_write;
+	  }
 	}
 	else if (egd_query[i].command == 2)
 	{
-	  egd_query[i].to_process = egd_query[i].secbyte;
-	  egd_query[i].yet_processed = 0;
 	  seed_internal(NULL);	/* Mix in just before query */
 	  seed_stat();
-	  if (rand_bytes(egd_query[i].buffer, egd_query[i].secbyte) > 0) {
-	    egd_query[i].state = wait_write;
-	    retrieved_bytes = 1;
-	  }
-	  else
-	    egd_query[i].state = wait_entropy;
+	  egd_query[i].to_process = egd_query[i].secbyte;
+	  egd_query[i].yet_processed = 0;
+	  egd_query[i].state = wait_entropy2;
 	}
 	else if (egd_query[i].command == 3)
 	{
@@ -637,18 +668,50 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	if (error <= 0)
 	  egd_query[i].state = to_be_error_closed;
 	else
+	{
 	  egd_query[i].yet_processed += error;
-	if (egd_query[i].yet_processed == egd_query[i].to_process) {
-	  egd_query[i].state = wait_command;
-	  (void)fprintf(stderr, "Sent %d bytes to service %d\n",
-		egd_query[i].to_process, egd_query[i].fd);
-		}
+	  if (egd_query[i].yet_processed == egd_query[i].to_process) {
+	    egd_query[i].state = wait_command;
+	    (void)fprintf(stderr, "Sent %d bytes to service %d\n",
+			  egd_query[i].to_process, egd_query[i].fd);
+	  }
+	}
 	break;
       default:
 	break;
       }
       }
     }
+
+    /*
+     * Now retrieve entropy for those processes waiting for it.
+     * Only retrieve one chunk of entropy, such that we don't hang on
+     * here too long. For fairness, we record the location of the last
+     * query-connection, so that we can continue with the next one.
+     *
+     * Calls to rand_bytes() need not be checked for failure, as the
+     * rand_pool() check in the loop condition is present.
+     */
+    for (i = 0; (i < max_query_old) && (rand_pool() > 0); i++) {
+      i1 = (prev_location + i) % max_query_old;
+      if (egd_query[i1].state == wait_entropy1) {
+	seed_internal(NULL);	/* Mix in just before query */
+	seed_stat();
+	rand_bytes(egd_query[i1].buffer + 1, egd_query[i1].secbyte);
+	egd_query[i1].state = wait_write;
+	prev_location = (i1 + 1) % max_query_old;
+	break;
+      }
+      if (egd_query[i1].state == wait_entropy2) {
+	seed_internal(NULL);	/* Mix in just before query */
+	seed_stat();
+	rand_bytes(egd_query[i1].buffer, egd_query[i1].secbyte);
+	egd_query[i1].state = wait_write;
+	prev_location = (i1 + 1) % max_query_old;
+	break;
+      }
+    }
+
     /*
      * Perform the final cleanup
      */
@@ -669,6 +732,10 @@ void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
 	count_openfd--;
 	/* Remove delivered entropy */
 	(void)memset(egd_query[i].buffer, 0, 256);
+	if (i < prev_location)
+	  prev_location--;	/* No need to wrap around, we only have to care
+				 * about fairness.
+				 */
 	if (i < max_query - 1)
 	{
 	  (void)memmove(egd_query + i, egd_query + max_query - 1,
